/**
	File name : stringMath.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
	Pseudo string (mainly arrays used as strings) because JS string do not have enough practical flexibility.
		Warning : myString[i]= X  does not work, when X= myString[i] does!
*/
function stringMath(){


	/* ---------------------------------------------------------------
		Public variable for the "class"
	*/
	this.baseFrStr= getBaseFr();
	
	
	/* ---------------------------------------------------------------
		Methods interface. 
	*/
	
	/* -----------------------------------------------------------------------
		Pure vector and string arithmetical operations done in base ten! So
		need base convertion when dealing with other bases.
	*/
	this.add= add;
	this.sAdd= sAdd;
	
	this.dgtProd= dgtProd;
	this.sProd= sProd;
	
	this.times= times;
	this.sTimes= sTimes;
	this.sPow= sPow;
	
	this.sub= sub;
	this.sSub= sSub;
	
	this.divis= divis;
	
	this.primitDiv= primitDiv;
	this.sDivis= sDivis;
	
	
	
	// Base convertion
	this.bTenToVectBin= bTenToVectBin;
	this.baseXtoTen= baseXtoTen;
	this.baseXAluTen= baseXAluTen;		// faster for shorter numbers, uses directly the ALU
	this.baseXToY= baseXToY;	
	
	this.baseConvAndSolve= baseConvAndSolve;
	this.exchgVect= exchgVect;
	
	this.getPwBase= getPwBase;
	
	
	
	
	// Basic vector string operations
	this.largestFrst= largestFrst;
	this.inflateString= inflateString;
	this.inflateStrToBits= inflateStrToBits;	
	this.cloneArray= cloneArray;
	this.vectToStr= vectToStr;
	this.strToVect= strToVect;
	this.vectNumPos= vectNumPos;
	this.nbToVect= nbToVect;
	this.vectToNb= vectToNb;
	this.getStorageBase= getStorageBase;
	this.getBaseFr= getBaseFr;
	this.writePwBaseLength= writePwBaseLength;
	this.binConvPwTwBAL= binConvPwTwBAL;
	this.binMapStringSz= binMapStringSz;
	this.canonicOperands= canonicOperands;
	this.rmHeadChars= rmHeadChars;
	this.zeroFiller= zeroFiller;
	this.strOFiller= strOFiller;
	this.binMapStringVect= binMapStringVect;
	this.buildCharToBin= buildCharToBin;
	this.buildBinToChar= buildBinToChar;
	this.buildBinConVect= buildBinConVect;
	this.wect= wect;
	this.wectLn= wectLn;



	/* ---------------------------------------------------------------
		Private variable for the "class"
	*/
	var crLn= kk.crLn;		// often handy/shorter to have it locally.	
	var bsTenStr= '0123456789';
	var rPwBase= getPwBase();	




	/* -----------------------------------------------------------------------
		Methods definitions
	*/	
	
	/**
		A safe (64 characters present as "1 byte" in most char sets) base to store crypted texts with
		64 ascii printable characters that will make unpronunçable string (they are visible, so need to avoid random distractors)
		
		# : reserved for end string in case of partial coding.
	*/
	function getStorageBase(){
		return 'bcdfghjklmnpqrstvwxyzBCDFGHJKLMNPQRSTVWXYZ0123456789+-/*?!$%&:_@';
	}; // getStorageBase

	/**
		Get the french characters base. Rem : to get within 128 chars upper cases with accents are not allowed,except the most current.
	*/
	function getBaseFr(){
		return '@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789©\r\t\n/\\-_,.;:~`\'´¨^°§+"*ç%&=?!(){}[]$£€¢<>¦| #¬äëïöüâêîôûàèùé«»ÊÉÈ';	
	}; // getBaseFr

	
	/**
		ASCII printable characters + a few char usefull in the considered context to get to 128 chars for 7 bits coding.
		http://www.theasciicode.com.ar/ascii-printable-characters/tilde-swung-dash-ascii-code-126.html
		Note : need a 128/7 = 18.2... = 19 characters password to fully initialise the 128 bits xor shift algo!
		Last char is only partially significant!
		Example : blablablablablabla#
	*/
	function getPwBase(){
		return ' !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~üéâäàçêëèïîìÄÉôöòûùÿÖÜ£«»ÁÂÀÛÙÊËÈ';
	}; // getPwBase



	/* ---------------------------------------------------------------------------------
		Fast construction of conversion vectors.
		!! Need to use these to replace the hard code one in jsEncryption. !!
	*/

	/**
	*/		
	function initVect(N){
		var bVect = new Array();
		for (var i=0; i<N; i++){
			bVect[i]= '';		
		}		
		return bVect;
	} // initVect

	/**
		Build a binary string mapping vector of N elements (by adding columns of bits from the right to the left)
	*/
	function binMapStringVect(N,nbCol){	
		var bVect= initVect(N);		
		for (var col=0;col<nbCol;col++){		
			var bitPond= Math.pow(2,col);	// bit ponderation = number of time to keep the same bit
			var keepBit= bitPond;
			var zerOn= true;	// true if the bit zero is to display
			
			for (var i=0; i<N; i++){
				if (zerOn){
					bVect[i]= '0' + bVect[i];
				}else{
					bVect[i]= '1' + bVect[i];
				}
										
				keepBit--;
				if (keepBit<=0){
					zerOn= !zerOn;
					keepBit= bitPond;
				}				
			} // for
		} // for	
		return bVect;
	} // binMapStringVect

	/**
		Build the character to binary-code convertion associative vector.
	*/
	function buildCharToBin(charsStr,bVect){	
		var sz= charsStr.length;
		var vect = new Array();		
		for (var i=0; i<sz; i++){
			vect[ charsStr.charAt(i) ]= bVect[i];
		}
		return vect;
	} // buildCharToBin

	/**
		Build the binary-code to character conversion associative vector.
	*/
	function buildBinToChar(bVect,charsStr){
		var sz= charsStr.length;
		var vect = new Array();		
		for (var i=0; i<sz; i++){
			vect[ bVect[i] ]= charsStr.charAt(i);
		}
		return vect;
	} // buildBinToChar

	/**
		Build the 2 binary conversion vectorsto have instant for single character in the given base and reciproc.
		with the corresponding binary code.
	*/
	function buildBinConVect(charsStr){
		var o= new Object();
		var N= charsStr.length;
		var binWordSz= binMapStringSz(charsStr);
		
		var bVect= binMapStringVect(N,binWordSz);
		o.bWordSz= binWordSz;
		o.chrToBin= buildCharToBin(charsStr,bVect);	// assoc array
		o.binToChr= buildBinToChar(bVect,charsStr);	// assoc array
		return o;
	} // buildBinConVect

	/**
		Check full associative array
	*/
	function assoArrChk(lbl,ray){
		for (var i in ray) {
			writeLn(lbl + i + ' , ' + ray[i]);
		}
	}; // assoArrChk

	/*
		End fast construction of conversion vectors.
	---------------------------------------------------------- */


	/**
	*/
	function writePwBaseLength(){
		var b= getPwBase();
		writeLn('PwBase Length='+b.length);
	}; // writePwBaseLength

	
	/**
		Return an associative array giving the numerical position of each character of charSet
	*/
	function vectNumPos(charSet){
		var sz= charSet.length;
		var v= new Array();
		for (var i=0;i<sz;i++){
			v[charSet.charAt(i)]= i;		
		}
		return v;
	} // vectNumPos
		
		
	/**
		Fast conversion for a base in 2^n into a binary vector.
		Binary convertion of a power 2 compatible base
		
		Input :
			wip : wordInPTwo base (normally a long password)
			ptb : power two compatible base (tested with getBaseFr(), in that case a base of 128 chars )
			
		Returns a vector holding the binary number.
	*/
	function binConvPowTwoBase(wip,ptb){
		var wipV= strToVect(wip);
		var npV= vectNumPos(ptb);
		var szSmlBV= binMapStringSz(ptb);
		var binTotV= new Array();		
		var sz= wip.length;			
		for (var i=0; i<sz; i++){
			var c= wip.charAt(i);
			var smlBinV= bTenToVectBin(npV[c], szSmlBV);
			for (var j=0; j<szSmlBV; j++){
				binTotV.push(smlBinV[j]);
			}
		}
		return binTotV;
	}; // binConvPowTwoBase
	
	/**
		Binary conversion from a power 2 Base with Adusted Length.
	
		Input :
			wip : wordInPTwo base (normally a long password)
			ptb : power two compatible base (tested with getBaseFr(), in that case a base of 128 chars ).
			sz : desired size.
			
		Returns a vector holding the binary number of size sz.
	*/
	function binConvPwTwBAL(wip,ptb,sz){
		var v= binConvPowTwoBase(wip,ptb);
		return adjustLength(v,sz);
	}; // binConvPwTwBAL
	
	
	/**
		Discriminates between smaller and larger.		
		o.lSz, o.sSz, o.lV, o.sV
	*/
	function canonicOperands(a,b){
		var o= new Object();	
		var aSz= a.length;
		var bSz= b.length;
		if (aSz>bSz){
			o.lSz= aSz; // longer size
			o.sSz= bSz; // shorter size
			o.lV= a;
			o.sV= b;
		}else{
			o.lSz= bSz;
			o.sSz= aSz;
			o.lV= b;
			o.sV= a;
		}
		return o;
	}; // canonicOperands
	
		
	/**
		Could also have a numerical approach :
			rD= String(Math.floor(totD / 10));
			nD= String(totD % 10);
	*/
	function reportAndNewDgt(totD){
		var p= new Object();			
		if (totD<10){
			p.rD= '0';				// reported digit
			p.nD= String(totD);		// new digit
		}else{
			var totDStr= String(totD);
			p.rD= totDStr.charAt(0);
			p.nD= totDStr.charAt(1);
		}
		return p;
	}; // reportAndNewDgt
	
	/**
		Returns a+b, sum of 2 arrays
	*/
	function add(a,b){
		var o= canonicOperands(a,b);	
		var sD= '';		// small number digit buffer
		var lD= '';		// large number digit buffer
		var sumNb= new Array();	// the sum number	
		var p= reportAndNewDgt(0);	
			
		var s=  o.sSz-1;	// index for the small vector.
		for (var l=o.lSz-1; l>=0; l--){
			lD= Number(o.lV[l]);
			
			if (s>=0){
				sD= Number(o.sV[s]);
			}else{
				sD= 0;
			}			
			var totD= lD + sD + Number(p.rD);	// total of the 2 digits + report						
			p= reportAndNewDgt(totD);			
			sumNb.unshift(p.nD);
			s--;
		} // for
		
		if (p.rD!=='0'){
			sumNb.unshift(p.rD);
		}
		return sumNb;
	} // add

	/**
		Just strings. Base 10 !!!
	*/
	function sAdd(a,b){
		var vA= strToVect(a);
		var vB= strToVect(b);
		var vTot= add(vA,vB);
		return vectToStr(vTot);
	}; // sAdd
	

	/**
		Base conversion and solve. To operate on numbers in any base.
	*/
	function baseConvAndSolve(x,y,bN,oprt){
		var ptCmbLimit= document.getElementById('bsCmbLimit');
		
		if (ptCmbLimit!==null){
			var bsCmbLimit= ptCmbLimit.value;
			var gotLim= bsCmbLimit==='ok';
		}else{
			var gotLim= false;
		}
		
		var diffLimit= 500; // difficulty limit
		var limError= 'The numbers are too large for the option Secure&Fast, but you can choose the option "Maybe Slow" to compute any length, the speed is length dependant, some still quite fast and other quite slow, try progressively and avoid single processors computer with single thread browser.';
	
		if (oprt!=='Mx'){
			var vX= strToVect(x);
			var vY= strToVect(y);		
				
			if (bN===bsTenStr){
				// clone
				var vXTen= cloneArray(vX);
				var vYTen= cloneArray(vY);
			}else{
				// Translates 2 bases bN  strings x and y into 2 vector in base 10
				var vXTen= baseXtoTen(vX, bN);
				var vYTen= baseXtoTen(vY, bN);
			}
		}

		switch (oprt){
			case '+':
				var vResult= add(vXTen,vYTen);
			break;			
			
			case '*':
				if (gotLim){
					var diffScale= vXTen.length + vYTen.length;
					if (diffScale<=diffLimit){
						var vResult= times(vXTen,vYTen);
					}else{
						return limError;
					}
				}else{
					var vResult= times(vXTen,vYTen);
				}					
			break;
			
			case '-':
				var vResult= sortAndSub(vXTen,vYTen);
			break;
			
			case '/':
				var obj= divis(vXTen,vYTen);
				return quotientResid( vectToStr(baseTenToX(obj.quotient, bN)), vectToStr(baseTenToX(obj.residue, bN)) );
			break;
						
			// Suppl operators.				
			case 'Mx':
				if (!gotLim){
					if (bN===bsTenStr){
						return sMultplProd(x);
					}else{
						return sMultplLitProd(x,bN);	// rem : does the base conversion inside
					}
				}else{
					return 'Please choose the option "Maybe Slow", as a warning, because this operation can take a while! Trying to guess in advance how long is not done at the reverse of the other operations.';
				}
			break;
			
			case '^':
				if (gotLim){
					var diffScale= vXTen.length * vectToNb(vYTen);	// difficulty scale .. complexity in log(x)*log(y)
					if (diffScale<=diffLimit){
						var vResult= vPowToStr(vXTen,vYTen);
					}else{
						return limError;
					}
				}else{
					var vResult= vPowToStr(vXTen,vYTen);
				}					
			break;
			
			case '!':
				if (gotLim){
					if ( vectToNb(vXTen) <= diffLimit){
						var vResult= vFacToStr(vXTen);
					}else{
						return limError;
					}
				}else{
					var vResult= vFacToStr(vXTen);
				}					
			break;
											
					
			default: writeLn('Error2 in baseConvSolve');
		} // switch
		
		if (bN===bsTenStr){
			var buffRes= vectToStr(vResult);	
		}else{
			var buffRes= vectToStr(baseTenToX(vResult, bN));			
		}
		
		
		if (ptCmbLimit!==null){
			// general calculator context, the result is for display
			return buffRes + crLn + crLn + 'Spaced string=' + spacedStr(buffRes);
		}else{
			// need a pure result for to do anything with it!
			return buffRes;
		}
	}; // baseConvAndSolve


	/**
	*/
	function spacedStr(s){
		var sz= s.length;
		var r= '';
		for (var i=0; i<sz; i++){
			r+= s.charAt(i)+' ';
		}
		return r;	
	}; // spacedStr


	/**
		Product by a single digit
		Input :
			a : array of digits (chars)
			dt : single digit char
	*/
	function dgtProd(a,dt){
		var aSz= a.length;
		var p= reportAndNewDgt(0);	
		var prodNb= new Array();	// the sum number	
		
		for (var i=aSz-1; i>=0; i--){
			var aD= Number(a[i]);	// a digit
		
			var prodD= Number(dt)*aD +  Number(p.rD);	// digit product + reported digit			
			p= reportAndNewDgt(prodD);			
			prodNb.unshift(p.nD);
		}
		
		if (p.rD!=='0'){
			prodNb.unshift(p.rD);
		}
		
		return prodNb;
	}; // dgtProd
	
	/**
		Just strings. except b
	*/
	function sProd(a,b){
		var vA= strToVect(a);
		var vProd= dgtProd(vA,b);
		return vectToStr(vProd);
	}; // sProd
		
	/**
		Returns a*b
	*/
	function times(a,b){
		var totProd= new Array();
		var o= canonicOperands(a,b);		
		var tenPower= 0;	// power ten  of the smallest number digits
		
		// Scan the smallest number
		for (var i=o.sSz-1; i>=0; i--){
			var multDgt= o.sV[i];			
			var curProd= dgtProd(o.lV,multDgt);
			var cProdStr= vectToStr(curProd);
			
			for (var p= 0;p<tenPower;p++){
				curProd.push('0');
			}
			
			if (i<o.sSz-1){
				totProd= add(totProd,curProd);
			}else{
				totProd= cloneArray(curProd);
			}			
			tenPower++;
		}
		return totProd;
	} // times

	/**
		Just strings.
	*/
	function sTimes(a,b){
		var vA= strToVect(a);
		var vB= strToVect(b);
		var vTot= times(vA,vB);
		return vectToStr(vTot);
	}; // sTimes


	/**
		Just digit strings.
		Input :
			a : a vector of string to try all the pairs combinations from
		Output :
			a string giving all combinations with the result.
	*/
	function sMultplProd(a){
		var separator= document.getElementById('baseTwoDspl').value;	// on text box 2 usages!
		writeLn('separator='+separator);
		
		var stringVectA= a.split(separator);
		var szA= stringVectA.length;
		
	
		var curDspl= '';
		for (var i=0; i<szA; i++){
			for (var j=i; j<szA; j++){
			
				var curStrI= stringVectA[i];
				var curStrJ= stringVectA[j];
				
				var curVectI= strToVect(curStrI);
				var curVectJ= strToVect(curStrJ);
				var curProd= times(curVectI,curVectJ);
				
				curDspl+= vectToStr(curVectI)+'*'+vectToStr(curVectJ)+'='+vectToStr(curProd)+crLn;
			}
		}
		
		return curDspl;
	}; // sMultplProd


	/**
		Just literal strings (would be ok for digit str too but keep short one)
		Input :
			rawStr : raw string with separators and substrings.
			bN : the current base.
		Output :
			A full result string.(all combinations)	.
	*/
	function sMultplLitProd(rawStr, bN){
		var separator= document.getElementById('baseTwoDspl').value;	// on text box 2 usages!
		writeLn('Separator='+separator);
		
		var stringVectA= rawStr.split(separator);
		var szA= stringVectA.length;
		var vectVectA= new Array();
		for (var i=0; i<szA; i++){
			vectVectA[i]= baseXtoTen(strToVect(stringVectA[i]),bN);
		}
	
		var curDspl= '';
		for (var i=0; i<szA; i++){
			for (var j=i; j<szA; j++){
				var curProd= times(vectVectA[i],vectVectA[j]);
				var curResStr= vectToStr(baseTenToX(curProd,bN));
				curDspl+= stringVectA[i]+'*'+stringVectA[j]+'='+curResStr+crLn;
			}
		}
		return curDspl;
	}; // sMultplLitProd


	/**
		vectPow
	*/
	function sPow(a,b){
		var vA= strToVect(a);
		var vB= strToVect(b);
		var vTot= vectPow(vA,vB);
		return vectToStr(vTot);
	}; // sPow
	
	function vPowToStr(vA,vB){
		var vTot= vectPow(vA,vB);
		return vectToStr(vTot);
	}; // vPowToStr
	
	
	/**
		Returns a power b
	*/
	function vectPow(a,b){
		var curProd= nbToVect(1);
		var cPower= vectToNb(b);
		for (var i=0; i<cPower; i++){
			curProd= times(curProd,a);
		}
		return curProd;
	}; // vectPow


	/**
	*/
	function sFactorial(a){
		var vA= strToVect(a);
		var vTot= vectFact(vA);
		return vectToStr(vTot);
	}; // sFactorial
	
	function vFacToStr(vA){
		var vTot= vectFact(vA);
		return vectToStr(vTot);
	}; // vFacToStr
	
	
	
		
	
	/**
		Returns a factorial
	*/
	function vectFact(a){
		var lastFactNb= vectToNb(a);
		var curProd= nbToVect(1);
		
		for (var i=2; i<=lastFactNb; i++){
			var curVectFactor= nbToVect(i);
			curProd= times(curProd,curVectFactor);
		}
		return curProd;
	}; // vectFact


	/**
		Inflates the given string by squaring it until it is at least equal to the bottom limit number
			Input :
				s : given string.
				botLim : bottom inflating limit of digits numbers for the result string
				gotModif : does global modification (reverse it) to have another kind of string
				
				// 	function baseTenToX(vTen, charSet){
	*/
	function inflateString(s,botLim,gotModif){
		var vS= strToVect(s);
		var bTenS= baseXtoTen(vS,rPwBase);
			
		var botLimTen= Math.round(botLim * Math.log(rPwBase.length)/Math.log(10));
		var maxLoop= 10;
		var i= 0;
		
		var vProd= cloneArray(bTenS);		
		while (vProd.length<botLimTen && i<maxLoop){
			vProd= times(vProd,bTenS);			
			i++;
		};
				
		if (gotModif==='ok'){
			vProd= reverseVect(vProd);
		}
		
		var bPwStr= baseTenToX(vProd,rPwBase);			
		var finalStr= vectToStr(bPwStr);
		return finalStr;
	}; // inflateString
	
	
	/**
	*/
	function reverseVect(origV){
		var revV= new Array();	// reversed vector
		j= 0;
		var sz= origV.length;
		for (var i=sz-1; i>0; i--){
			revV[j]= origV[i];
			j++;
		}
		return revV;
	}; // reverseVect
	
	
	/**
		Inflates or trunk the string str1 to a 128 bits representation.
		Only for passwords : rPwBase.
	*/
	function inflateStrToBits(str,botLim,gotModif){
		var str2= inflateString(str,botLim,gotModif);
		return binConvPwTwBAL(str2,rPwBase,128);
	}; // inflateStrToBits


	/**
		Returns lV-sV (operands are already sorted by size, because sub() need to be used in the division where
		the operands are already sorted! (don't want to waste time trying to sort them again), so : calls to this
		method also need to sort the operands before).
		
		baseConvAndSolve()
		
			Input :
				lV : longuest vector
				sV : shortest vector
						
		o.lSz, o.sSz, o.lV, o.sV
		var o= canonicOperands(a,b);	
	*/
	function sub(lV,sV){
		lSz= lV.length;
		sSz= sV.length;
	
		var sD= '';		// small number digit buffer
		var lD= '';		// large number digit buffer
		var nD= 0;	// new digit
		var rD= 0;	// reported digit
		var subNb= new Array();	// the buffer for the final result	
			
		var s=  sSz-1;	// index for the small vector.
		for (var l=lSz-1; l>=0; l--){
			lD= Number(lV[l]);	// large number digit
			if (s>=0){	
				sD= Number(sV[s]);	// small number digit
			}else{
				sD= 0;
			}
						
			if (lD>=(sD+rD)){
				nD= lD-(sD+rD);
				rD= 0;
			}else{
				nD= 10+lD-sD-rD;				
				rD= 1;
			}
			
			subNb.unshift(String(nD));
			s--;
		} // for	
		
		subNb= remHdZeros(subNb)	;
		
		if (rD!==0){
			subNb.unshift(String(rD));
		}
		
		return subNb;
	} // sub

	/**
		o.lSz, o.sSz, o.lV, o.sV
		var o= canonicOperands(a,b);	
	*/
	function sortAndSub(a,b){
		var o= canonicOperands(a,b);	
		return sub(o.lV, o.sV);
	} // sortAndSub

	/**
	*/
	function sSub(a,b){
		var vA= strToVect(a);
		var vB= strToVect(b);		
		var vSub= sortAndSub(vA,vB);
		return vectToStr(vSub);
	}; // sSub


	/**
		removeHeadZeros
	*/
	function remHdZeros(v){
		while(v.length>0 && v[0]==='0'){
			v.shift();
		}
		return v;
	}; // remHdZeros
	

	/**
		remove N Head Chars of V
	*/
	function rmHeadChars(v,N){
		for (var i=0; i<N; i++){
			v.shift();
		}
		return v;
	}; // rmHeadChars
	

	/**
		Returns a/b
			Input :
				lV : large vector number
				divisor : small vector number
			Output :
				Return o.quotient; o.residue;	// divis(lV,divisor)
	*/
	function divis(lV,divisor){
		var smallDdd= new Array();	// small Dividend
		var totQuot= new Array();	// total Quotient
		
		var gotFirstDigit= false;
		var sz= lV.length;
		for (var i=0; i<sz; i++){				
			var charLV= lV[i];
			smallDdd.push(charLV);			
		
			var lgF= largestFrst(smallDdd,divisor);
			if (lgF===1 || lgF===0){
				// can divide smallDdd / divisor
				var pQuo= primitDiv(smallDdd, divisor);	// primitive quotient
				totQuot.push(String(pQuo.quotient));
				smallDdd= cloneArray(pQuo.residue);
				gotFirstDigit= true;
			}else{
				// can't divide : smallDdd is too small				
				if (gotFirstDigit){		// avoid early zeros.
					totQuot.push('0');
				}
			}
		} // for
				
		return quotientResid(totQuot,smallDdd);
	} // divis
	
	/**
		Build and return a quotient and residue object.
		Input :
			quo : quotien (array)
			resid : residue (array)
		// o.quotient	//	o.residue
	*/
	function quotientResid(quo,resid){
		var o= new Object();	
		o.quotient= quo;
		o.residue= resid;
		return o;
	}; // quotientResid
	

	/**
		stringMath.sDivis(a,b)
	*/
	function sDivis(a,b){
		var vA= strToVect(a);
		var vB= strToVect(b);
		var rDiv= divis(vA,vB);
		
		var result= new Object();
		result.quotient= vectToStr(rDiv.quotient);
		result.residue= vectToStr(rDiv.residue);
		return result;
	}; // sDivis


	/**
		Returns true if the smaller number-vector is the first number.
		Comparaison done in base ten.
	*/
	function largestFrst(a,b){
		remZerPfx(a);
		remZerPfx(b);	
	
		aSz= a.length;
		bSz= b.length;

		if (aSz>bSz) return 1;		// true
		if (aSz<bSz) return -1;	// false
				
		// size equality so need to compare digits to digits from left to right		
		for (var i=0; i<aSz; i++){
			var aNb= Number(a[i]);
			var bNb= Number(b[i]);
			
			if (aNb > bNb) return 1;
			if (aNb<bNb) return -1;
		}
		
		return 0;	// strict equality
	}; // largestFrst
	
	
	/**
		Remonve zero prefix!.
	*/
	function remZerPfx(v){	
		var sz= v.length;
		var i=0;
		while (v[i]==='0' && i<sz-1){
			v.shift()
			i++;
		}
	}; // remZerPfx
		

	/**
		Primitive division : uses successive substraction between vectors to find out 
		quotient and residue.
	*/
	function primitDiv(lV,sV){
		var nLv= cloneArray(lV);	// new large vector
		var res= new Object();		// result
		var i= 0;
		while(largestFrst(nLv,sV)!==-1){
			nLv= sub(nLv,sV);
			i++;
		}
		res.quotient= i;
		res.residue= nLv;
		return res;
	}; // primitDiv
	
	
	/**
		Exchange 2 vector cells
	*/
	function exchgVect(v,i,j){
		var a= v[i];
		v[i]= v[j];
		v[j]= a;
	}; // exchgVect
	

	/* ---------------------------------------------------------------------
		Full string base conversion - Start.
	*/

	/**
		Take a string in base 10 and return the equivalent in base X.
		Input :
			vTen : (vector)number to transform given in base X
			x : decimal numerical span of the new base (binary=2, hexa=16 etc..)
			charSet : char set of the new base.
	*/
	function baseTenToX(vTen, charSet){
		var x= nbToVect(charSet.length)		
		var buffRes= new Array();	// buffer result
			
		var result= quotientResid(vTen,new Array());		
		do{	
			result= divis(result.quotient, x);
			var indx= vectToNb(result.residue);
			buffRes.unshift( charSet.charAt(indx) );	
		}while(result.quotient.length>0);	
		
		return buffRes;
	} // baseTenToX	
	

	/**
		Return an array / vector
			Input :
				nbTen : number in base ten to convert.
				sz : the size of the final vector, with the 1st left bit considered as the left most bit of a positive number.
			Output :
				the binary vector match of sz length.
	*/
	function bTenToVectBin(nbTen, sz){
		var vNbTen= nbToVect(nbTen);
		var vBin= baseTenToX(vNbTen, '01');	// binary vector		
		return adjustLength(vBin,sz);		
	}; // bTenToVectBin	


	/**
		Adjust the length of the given vector,
			Input :
				v : vector to adapt the size with prefixed zeros or truncation.
				sz : desired size of v.
			Return : 
				resV the adjusted result vector.
	*/
	function adjustLength(v,sz){
		var vSz= v.length;	// binary vector size
		
		if (vSz===sz){
			return v;
		}		 		
		
		if (vSz < sz){
			var delta= sz-vSz;		 	
			zVect= zeroFiller(delta);
			var resV= zVect.concat(v);	// result vector
		}else{
			// (vSz > sz)
			var resV= v.slice(0,sz);
		}

		return resV; 
	}; // adjustLength
	
	/**
	*/
	function zeroFiller(sz){
		var v= new Array();
		for (var i=0; i<sz; i++){
			v.push('0');
		}
		return v;
	}; // zeroFiller
	
	/**
		Str Zeros filler! O char so dbl clisk copy all!
	*/
	function strOFiller(sz){
		return vectToStr(zeroFiller(sz));
	}; // strOFiller


	/**
		Take a vector in base X and returns the equivalent in base 10.
			Input :
				vInX : (a vector) number given in base X to transform in base ten.
				charSet : (a string) char set of the new base.
			Output :
				Returns a standard javascrip/ALU number
	*/
	function baseXtoTen(vInX, charSet){
		var xSz= nbToVect(charSet.length);	// decimal numerical value of the new base.
		var buffStr= '';
		var numPos= vectNumPos(charSet);
		
		var nbSz= vInX.length;
		var p= 0;	// power
		var i= nbSz-p-1;
		
		var powerX= nbToVect(1);	// Vector of value one in vase 10
		var totNb= nbToVect(0);		// Vector of value 0 in vase 10		
			
		while (p<nbSz){
			var c= vInX[i];		// Current character from right to left;
			var np= numPos[c];	// Number matching the char		
			if (kk.isNotUndef(np)){
				if (np<10){
					var curProdV= dgtProd(powerX,np);
				}else{
					var curProdV= times(powerX, nbToVect(np) );
				}							
				totNb= add(totNb,curProdV);
			}else{
				writeLn('Warning:"'+c+'" is not in the set of characters of Base 1');
			}
			
			powerX= times(powerX, xSz );
			i--;
			p++;
		}		
		return totNb;
	} // baseXtoTen
	
	/**
		For small string, much faster than baseXtoTen() of course.
		Take a string in base X and returns the equivalent in base 10.
		Input :
			nbInX : number (a vector) to transform given in base X
			charSet : char set of the new base.
		Ouputs ;
			The converted number expressed as a string
	*/
	function baseXAluTen(nbInX, charSet){
		var x= charSet.length;	// decimal numerical value of the new base.
		var buffStr= '';
		var numPos= vectNumPos(charSet);	// leave it local for the sake of generalisation
		
		var nbSz= nbInX.length;
		var p= 0;	// power
		var i= nbSz-p-1;
		
		var powerX= 1;
		var totNb= 0;		
		while (p<nbSz){
			var c= nbInX[i];	// current character from right to left;
			var np= numPos[c];
			
			if (kk.isNotUndef(np)){
				totNb+= powerX*np;
			}else{
				writeLn('Warning:"'+c+'" is not in the set of characters of Base 1');
			}
			
			powerX= powerX*x;			
			i--;
			p++;
		}		
		return totNb;
	} // baseXAluTen
	

	/**
	*/
	function baseXToY(nbX,setX,setY){	
		var nbTen= baseXtoTen(nbX,setX);		
		var nbY= baseTenToX(nbTen,setY);
		return nbY;
	} // baseXToY

	/*
		Full string base conversion - End.
	--------------------------------------------------------------------- */



	/* -------------------------------------------------------------
		Basic array / strings methods
	*/


	/**
		Clone a single level array, for multi levels, could have a recursive function.
	*/
	function cloneArray(a){
		return kk.cloneArray(a);
	}; // cloneArray
	

	/**
		Exchange 2 vector cells
	*/
	function exchgVect(v,i,j){
		var a= v[i];
		v[i]= v[j];
		v[j]= a;
	}; // exchgVect


	/**
	*/
	function invertStr(s){
		var nStr= '';
		var sz= s.length;
		for (var i=sz-1; i>0; i--){
			nStr+= s.charAt(i);
		}
		return nStr;
	};
		

	/**
		String character exchange (should be AVOIDED for processing intensive situations), 
		do the vector conversion as soon as possible.
		Warning on Javascript problems : 
			v.toString();		// a native method that adds ','
			s.toArray();		// NADA, this methods is not native.
			s='ok'; s[i]= 'A';	// s[i] does not work, just ignored! Not even an error or a parallel vector load.
		Advice for speed do the vector transformation as early as possible and stay in vector format.
	*/
	function exchgStr(s,i,j){
		var v= strToVect(s);
		exchgVect(v,i,j);		
		return vectToStr(v);
	} // exchgStr
		
	function strToVect(s){
		return kk.bStrToVect(s);	// byte string !!!
	} // strToVect
	
	function vectToStr(v){
		return kk.vectToStr(v);
	} // vectToStr
	
	
	/**
		Transform a decimal number into a vector-array
	*/
	function nbToVect(n){
		return strToVect(String(n));	
	}; // nbToVect
		
	/**
		Transform a vector-array into a decimal number.
	*/
	function vectToNb(v){
		return Number(vectToStr(v));	
	}; // vectToNb
	
			
	/**
		String character replacement (should be AVOIDED for processing intensive situations). 
		Altern: (not tested) String.prototype.replaceAt=function(i,c){return this.substr(0,i)+c+this.substr(i+1);}		
		Do the vector conversion as soon as possible.
	*/
	function replaceAt(s,i,c){
		var vect= strToVect(s);		
		vect[i]= c;	
		return vectToStr(vect);
	} // replaceAt
	


	/**
		Write a Vector content.
	*/
	function wect(lbl,v){
		write(lbl+vectToStr(v));
	}; // wect

	/**
	*/
	function wectLn(lbl,v){
		writeLn(lbl+vectToStr(v));
	}; // wectLn



	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}





		
	/**
		Return the size of a binary string used to code the char set.		
		Input :
			sv string or vector holding the char set.
			
		Remark : Math.round would work only for N² sizes
	*/
	function binMapStringSz(sv){
		return Math.ceil(logX(sv.length,2)-0.00001);
	} // binMapStringSz
	
	/**
		Return a logarythm of N in base X
	*/
	function logX(N,x){
		return Math.log(N)/Math.log(x);
	} // logX




	/**
	*/
	function chabl(){

	}; // chabl
	

} // stringMath
