<?php
/**
	File name : pseudoRnd.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	A class to generate a pseudo random vector to crypt many strings.
*/
class pseudoRnd{
	

	function __construct(&$top){
		$this->top= &$top;
	} // construct

	function setMain(&$main){
		$this->main= &$main;
	} // setMain

	
	/**
		Returns true if we got to use large integers for crypting
	*/
	function usesLargeCryptInt($sortOfInt){
		if ($sortOfInt==='def'){
			// Uses the default impose by the global constant defined inside the connection code.
			return _c_forceLargeInt===_c_ok;
		}else{
			// Uses the sort of integer imposed by the $sortOfInt parameter
			switch ($sortOfInt){
				case 'small':
					return false;
				break;
				
				case 'large':
					return true;
				break;
				
				default: 
					print('Error in pseudoRnd::usesLargeCryptInt'._c_);
					return _c_forceLargeInt===_c_ok;
			} // switch
		}
	} // usesLargeCryptInt
	

	/**
		Convert the small large integer notation to explicit text 'small' / 'large'
	*/
	function smalLargeIntToExplicit($kCode){
		switch ($kCode){
			case _c_ok:
				return 'large';
			break;
			
			case _c_ko:
			default: 
				return 'small';
			break;
		} // switch
	} // smalLargeIntToExplicit
	

	/**
		Multiplicative and Sum of an hexadecimal seeding string (transformed password etc) (to have a bit of space and minimize twin possibilities), compound the characters 
		of a string $s. Also see function serieMCG.
		
		With small integers (some lazy servers) :
			Z(i) = [16807 * Z(i-1)] mod 2147483647
			Considering minimum maxInt of 2147483647,
			We have a possible seed-span of :
				2147483647 / 16807 === 127773.1	  (that is tiny)
		See n serieMCG()
		
		Input :
			$str : the hexadecimal string to convert (inbetween converted to an hexadecimal string) to an integer $sum
			$sortOfInt : the sort of integers
		Output :
			$sum : the sum derived from $str and used generate a seed to ... generate a random serie to crypt anything.
	*/
	function sumHexStr($str,$sortOfInt){
		$cSp=16; // the space of the set of characters (utf-8 and large codes are coherced into hexadecimal then 16 is used)
		$hxSz= strlen($str);	// number of byte chars!
		
		$sum= 0;
		if ($this->usesLargeCryptInt($sortOfInt)){
			if ($hxSz>34){
				$hxSz= 34;
			}
			$y1= pow(2,40) / $cSp;
		}else{
			if ($hxSz>11){
				$hxSz= 11;
			}
			$y1= 127773 / $cSp;
		}
		
		$y= pow($y1,(1/$hxSz));	
		if ($y>= $cSp){
			$base= $cSp;
		}else{
			$base= floor($y);
		}
		
		$spacer= $base;
		for($i=0;$i<$hxSz;$i++){
			$hexChar= substr($str,$i,1);
			$hx= $this->hexToInt($hexChar);
			$sum= $sum+$spacer*$hx;
			$spacer= $spacer*$base;
		}
		return $sum;
	} // sumHexStr
	

	/* -----------------------------------------------------------------------------
	
		MCG : Multiplicative Congruential Generators
		
		Start the serie at the password numerical value.
		
		Z(i) = (7^5 *  Z(i-1) ) mod (2^31 -1)
		Z(i) = [16807 * Z(i-1)] mod 2147483647
		
		Input :
			$seed : the seed for the serie (where we tap inside the large loop of integers).
			$sz : size of the serie of pseudo random numbers.
			
		Call :
			genAsa.serieMCG(5000,100);
			
		Exmpl : $rndV= $this->serieMCG($sumKey,$lnVect);
	*/
	function serieMCG($seed,$sz,$sortOfInt, $cSpace=16){
		if ($this->usesLargeCryptInt($sortOfInt)){
			return $this->serMCGLrgInt($seed,$sz,$cSpace);
		}else{
			return $this->serMCGSmlInt($seed,$sz,$cSpace);
		}
	} // serieMCG


	/**
		Server MCG large integer random vector, tailored to a $cSpace integer span!
		
		Remark : would be too complicated to do the full crypt inside the loop, so need a vector, but some challenge will
		be too long, so, we need to keep going the serie  (without risking to take the same, so internal values must be kept!)
	*/
	function serMCGLrgInt($seed,$sz, $cSpace=16){
		$v= array();
		$z= $seed;
		for ($i=0; $i<$sz; $i++){
			$prod= 16807*$z;
			$z= $prod % 2147483647;
			if ($z===0){$z= 1;}	// Avoid rare stalling
			$v[$i]= $z % $cSpace;
		}
		
		// loads the returned array
		$rAr['v']= &$v;
		$rAr['seed']= $z;
		return $rAr;
	} // serMCGLrgInt


	/**
		Server MCG small integer random vector, tailored to a $cSpace integer span!
	*/
	function serMCGSmlInt($seed,$sz, $cSpace=16){
		$v= array();
		$z= $seed;
		for ($i=0; $i<$sz; $i++){
			$prod= 211*$z;
			$z= $prod % 9870967;	// Smaller prime
			if ($z===0){$z= 1;}	// Avoid rare stalling
			$v[$i]= $z % $cSpace;
		}
		
		$rAr['v']= &$v;
		$rAr['seed']= $z;
		return $rAr;
	} // serMCGSmlInt


	/**
		Hexadecimal code converted to an integer for a single element!
		Input :
			$h ; a hexadecimal character
		Output :
			The integer match of h 
			
		Also see mbCrypt::intToHex()
	*/
	function hexToInt($h){
		$main= &$this->main;
	
		if ( $main->isSimpleNumber($h) ){
			$n= $h;
		}else{
			$n= 10 + ord($h)-ord('a');	// need to make sure it's lower cases
		}
		return (int)$n;
	} // hexToInt
	

	/**
		Input :
			$c : a byte character
		Output :
			$h0, $h1 : hexadecimal match of $c
	*/
	function byteCharToHexN($c,&$h0,&$h1){
		$hexStr= bin2hex($c);
		$h0= $hexStr[0];	// $h0= $this->hexToInt($h0);
		$h1= $hexStr[1];	// $h1= $this->hexToInt($h1);
	} // byteCharToHexN


	/**
	*/
	function chabl(){
	} // chabl

} // pseudoRnd
?>
